@@ -25,6 +25,8 @@ import org.apache.bookkeeper.client.BookKeeper;
 import org.apache.bookkeeper.mledger.AsyncCallbacks.CloseCallback;
 import org.apache.bookkeeper.mledger.AsyncCallbacks.DeleteCursorCallback;
 import org.apache.bookkeeper.mledger.AsyncCallbacks.MarkDeleteCallback;
+import org.apache.bookkeeper.mledger.Position;
+import org.apache.bookkeeper.mledger.PositionFactory;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.pulsar.common.api.proto.CommandSubscribe;
 import org.slf4j.Logger;
@@ -35,7 +37,7 @@ public class NonDurableCursorImpl extends ManagedCursorImpl {
     private final boolean readCompacted;
 
     NonDurableCursorImpl(BookKeeper bookkeeper, ManagedLedgerImpl ledger, String cursorName,
-                         PositionImpl startCursorPosition, CommandSubscribe.InitialPosition initialPosition,
+                         Position startCursorPosition, CommandSubscribe.InitialPosition initialPosition,
                          boolean isReadCompacted) {
         super(bookkeeper, ledger, cursorName);
         this.readCompacted = isReadCompacted;
@@ -53,7 +55,7 @@ public class NonDurableCursorImpl extends ManagedCursorImpl {
                     initializeCursorPosition(ledger.getFirstPositionAndCounter());
                     break;
             }
-        } else if (startCursorPosition.getLedgerId() == PositionImpl.EARLIEST.getLedgerId()) {
+        } else if (startCursorPosition.getLedgerId() == PositionFactory.EARLIEST.getLedgerId()) {
             // Start from invalid ledger to read from first available entry
             recoverCursor(ledger.getPreviousPosition(ledger.getFirstPosition()));
         } else {
@@ -66,8 +68,8 @@ public class NonDurableCursorImpl extends ManagedCursorImpl {
                 readPosition, markDeletePosition);
     }
 
-    private void recoverCursor(PositionImpl mdPosition) {
-        Pair<PositionImpl, Long> lastEntryAndCounter = ledger.getLastPositionAndCounter();
+    private void recoverCursor(Position mdPosition) {
+        Pair<Position, Long> lastEntryAndCounter = ledger.getLastPositionAndCounter();
         this.readPosition = isReadCompacted() ? mdPosition.getNext() : ledger.getNextValidPosition(mdPosition);
         markDeletePosition = ledger.getPreviousPosition(this.readPosition);
 
@@ -96,7 +98,7 @@ public class NonDurableCursorImpl extends ManagedCursorImpl {
     }
 
     @Override
-    protected void internalAsyncMarkDelete(final PositionImpl newPosition, Map<String, Long> properties,
+    protected void internalAsyncMarkDelete(final Position newPosition, Map<String, Long> properties,
             final MarkDeleteCallback callback, final Object ctx) {
         // Bypass persistence of mark-delete position and individually deleted messages info
 
@@ -111,8 +113,6 @@ public class NonDurableCursorImpl extends ManagedCursorImpl {
     @Override
     public void asyncClose(CloseCallback callback, Object ctx) {
         STATE_UPDATER.set(this, State.Closed);
-        closeWaitingCursor();
-        setInactive();
         callback.closeComplete(ctx);
     }
 
