@@ -51,6 +51,7 @@ import org.apache.bookkeeper.mledger.ManagedLedgerException;
 import org.apache.bookkeeper.mledger.ManagedLedgerFactory;
 import org.apache.bookkeeper.mledger.ManagedLedgerFactoryConfig;
 import org.apache.bookkeeper.mledger.Position;
+import org.apache.bookkeeper.mledger.PositionFactory;
 import org.apache.bookkeeper.test.MockedBookKeeperTestCase;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -65,7 +66,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
     void readFromEmptyLedger() throws Exception {
         ManagedLedger ledger = factory.open("my_test_ledger");
 
-        ManagedCursor c1 = ledger.newNonDurableCursor(PositionImpl.EARLIEST);
+        ManagedCursor c1 = ledger.newNonDurableCursor(PositionFactory.EARLIEST);
         List<Entry> entries = c1.readEntries(10);
         assertEquals(entries.size(), 0);
         entries.forEach(Entry::release);
@@ -89,14 +90,14 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
         ManagedLedger ledger = factory.open("non_durable_cursor_at_non_existent_msgid");
         ManagedLedgerImpl mlImpl = (ManagedLedgerImpl) ledger;
 
-        PositionImpl position = mlImpl.getLastPosition();
+        Position position = mlImpl.getLastPosition();
 
-        ManagedCursor c1 = ledger.newNonDurableCursor(new PositionImpl(
+        ManagedCursor c1 = ledger.newNonDurableCursor(PositionFactory.create(
             position.getLedgerId(),
             position.getEntryId() - 1
         ));
 
-        assertEquals(c1.getReadPosition(), new PositionImpl(
+        assertEquals(c1.getReadPosition(), PositionFactory.create(
             position.getLedgerId(),
             0
         ));
@@ -109,7 +110,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
     void testZNodeBypassed() throws Exception {
         ManagedLedger ledger = factory.open("my_test_ledger");
 
-        ManagedCursor c1 = ledger.newNonDurableCursor(PositionImpl.EARLIEST);
+        ManagedCursor c1 = ledger.newNonDurableCursor(PositionFactory.EARLIEST);
         assertTrue(ledger.getCursors().iterator().hasNext());
 
         c1.close();
@@ -125,8 +126,8 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
         ManagedLedger ledger = factory.open("my_test_ledger",
                 new ManagedLedgerConfig().setRetentionTime(1, TimeUnit.HOURS).setRetentionSizeInMB(1));
 
-        ManagedCursor c1 = ledger.newNonDurableCursor(PositionImpl.LATEST);
-        ManagedCursor c2 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c1 = ledger.newNonDurableCursor(PositionFactory.LATEST);
+        ManagedCursor c2 = ledger.newNonDurableCursor(PositionFactory.LATEST);
 
         ledger.addEntry("entry-1".getBytes(Encoding));
         ledger.addEntry("entry-2".getBytes(Encoding));
@@ -158,8 +159,8 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
         ManagedLedger ledger = factory.open("my_test_ledger", new ManagedLedgerConfig().setMaxEntriesPerLedger(1)
                 .setRetentionTime(1, TimeUnit.HOURS).setRetentionSizeInMB(1));
 
-        ManagedCursor c1 = ledger.newNonDurableCursor(PositionImpl.LATEST);
-        ManagedCursor c2 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c1 = ledger.newNonDurableCursor(PositionFactory.LATEST);
+        ManagedCursor c2 = ledger.newNonDurableCursor(PositionFactory.LATEST);
 
         ledger.addEntry("entry-1".getBytes(Encoding));
         ledger.addEntry("entry-2".getBytes(Encoding));
@@ -188,7 +189,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
         ManagedLedger ledger = factory.open("my_test_ledger", new ManagedLedgerConfig().setMaxEntriesPerLedger(1)
                 .setRetentionTime(1, TimeUnit.HOURS).setRetentionSizeInMB(1));
 
-        ManagedCursor c1 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c1 = ledger.newNonDurableCursor(PositionFactory.LATEST);
 
         ledger.close();
 
@@ -205,15 +206,15 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
         ManagedLedger ledger = factory.open("my_test_ledger", new ManagedLedgerConfig().setMaxEntriesPerLedger(2)
                 .setRetentionTime(1, TimeUnit.HOURS).setRetentionSizeInMB(1));
 
-        ManagedCursor c1 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c1 = ledger.newNonDurableCursor(PositionFactory.LATEST);
         ledger.addEntry("dummy-entry-1".getBytes(Encoding));
-        ManagedCursor c2 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c2 = ledger.newNonDurableCursor(PositionFactory.LATEST);
         ledger.addEntry("dummy-entry-2".getBytes(Encoding));
-        ManagedCursor c3 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c3 = ledger.newNonDurableCursor(PositionFactory.LATEST);
         ledger.addEntry("dummy-entry-3".getBytes(Encoding));
-        ManagedCursor c4 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c4 = ledger.newNonDurableCursor(PositionFactory.LATEST);
         ledger.addEntry("dummy-entry-4".getBytes(Encoding));
-        ManagedCursor c5 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c5 = ledger.newNonDurableCursor(PositionFactory.LATEST);
 
         assertEquals(c1.getNumberOfEntries(), 4);
         assertTrue(c1.hasMoreEntries());
@@ -242,15 +243,15 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
         ManagedLedger ledger = factory.open("my_test_ledger", new ManagedLedgerConfig().setMaxEntriesPerLedger(2)
                 .setRetentionTime(1, TimeUnit.HOURS).setRetentionSizeInMB(1));
 
-        ManagedCursor c1 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c1 = ledger.newNonDurableCursor(PositionFactory.LATEST);
         Position p1 = ledger.addEntry("dummy-entry-1".getBytes(Encoding));
-        ManagedCursor c2 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c2 = ledger.newNonDurableCursor(PositionFactory.LATEST);
         ledger.addEntry("dummy-entry-2".getBytes(Encoding));
-        ManagedCursor c3 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c3 = ledger.newNonDurableCursor(PositionFactory.LATEST);
         Position p3 = ledger.addEntry("dummy-entry-3".getBytes(Encoding));
-        ManagedCursor c4 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c4 = ledger.newNonDurableCursor(PositionFactory.LATEST);
         Position p4 = ledger.addEntry("dummy-entry-4".getBytes(Encoding));
-        ManagedCursor c5 = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor c5 = ledger.newNonDurableCursor(PositionFactory.LATEST);
 
         assertEquals(c1.getNumberOfEntriesInBacklog(false), 4);
         assertEquals(c2.getNumberOfEntriesInBacklog(false), 3);
@@ -334,7 +335,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
     @Test(timeOut = 20000)
     void markDeleteGreaterThanLastConfirmedEntry() throws Exception {
         ManagedLedger ml1 = factory.open("my_test_ledger");
-        ManagedCursor mc1 = ml1.newNonDurableCursor(PositionImpl.get(Long.MAX_VALUE - 1, Long.MAX_VALUE - 1));
+        ManagedCursor mc1 = ml1.newNonDurableCursor(PositionFactory.create(Long.MAX_VALUE - 1, Long.MAX_VALUE - 1));
         assertEquals(mc1.getMarkDeletedPosition(), ml1.getLastConfirmedEntry());
     }
 
@@ -342,13 +343,13 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
     void testResetCursor() throws Exception {
         ManagedLedger ledger = factory.open("my_test_move_cursor_ledger",
                 new ManagedLedgerConfig().setMaxEntriesPerLedger(10));
-        ManagedCursor cursor = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor cursor = ledger.newNonDurableCursor(PositionFactory.LATEST);
         ledger.addEntry("dummy-entry-1".getBytes(Encoding));
         ledger.addEntry("dummy-entry-2".getBytes(Encoding));
         ledger.addEntry("dummy-entry-3".getBytes(Encoding));
-        PositionImpl lastPosition = (PositionImpl) ledger.addEntry("dummy-entry-4".getBytes(Encoding));
+        Position lastPosition = ledger.addEntry("dummy-entry-4".getBytes(Encoding));
         final AtomicBoolean moveStatus = new AtomicBoolean(false);
-        PositionImpl resetPosition = new PositionImpl(lastPosition.getLedgerId(), lastPosition.getEntryId() - 2);
+        Position resetPosition = PositionFactory.create(lastPosition.getLedgerId(), lastPosition.getEntryId() - 2);
         try {
             cursor.resetCursor(resetPosition);
             moveStatus.set(true);
@@ -366,14 +367,14 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
     void testasyncResetCursor() throws Exception {
         ManagedLedger ledger = factory.open("my_test_move_cursor_ledger",
                 new ManagedLedgerConfig().setMaxEntriesPerLedger(10));
-        ManagedCursor cursor = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor cursor = ledger.newNonDurableCursor(PositionFactory.LATEST);
         ledger.addEntry("dummy-entry-1".getBytes(Encoding));
         ledger.addEntry("dummy-entry-2".getBytes(Encoding));
         ledger.addEntry("dummy-entry-3".getBytes(Encoding));
-        PositionImpl lastPosition = (PositionImpl) ledger.addEntry("dummy-entry-4".getBytes(Encoding));
+        Position lastPosition = ledger.addEntry("dummy-entry-4".getBytes(Encoding));
         final AtomicBoolean moveStatus = new AtomicBoolean(false);
         CountDownLatch countDownLatch = new CountDownLatch(1);
-        PositionImpl resetPosition = new PositionImpl(lastPosition.getLedgerId(), lastPosition.getEntryId() - 2);
+        Position resetPosition = PositionFactory.create(lastPosition.getLedgerId(), lastPosition.getEntryId() - 2);
 
         cursor.asyncResetCursor(resetPosition, false, new AsyncCallbacks.ResetCursorCallback() {
             @Override
@@ -399,7 +400,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
     void rewind() throws Exception {
         ManagedLedger ledger = factory.open("my_test_ledger", new ManagedLedgerConfig().setMaxEntriesPerLedger(2)
                 .setRetentionTime(1, TimeUnit.HOURS).setRetentionSizeInMB(1));
-        ManagedCursor c1 = ledger.newNonDurableCursor(PositionImpl.EARLIEST);
+        ManagedCursor c1 = ledger.newNonDurableCursor(PositionFactory.EARLIEST);
         Position p1 = ledger.addEntry("dummy-entry-1".getBytes(Encoding));
         Position p2 = ledger.addEntry("dummy-entry-2".getBytes(Encoding));
         Position p3 = ledger.addEntry("dummy-entry-3".getBytes(Encoding));
@@ -450,11 +451,11 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
     @Test(timeOut = 20000)
     void markDeleteSkippingMessage() throws Exception {
         ManagedLedger ledger = factory.open("my_test_ledger", new ManagedLedgerConfig().setMaxEntriesPerLedger(10));
-        ManagedCursor cursor = ledger.newNonDurableCursor(PositionImpl.EARLIEST);
+        ManagedCursor cursor = ledger.newNonDurableCursor(PositionFactory.EARLIEST);
         Position p1 = ledger.addEntry("dummy-entry-1".getBytes(Encoding));
         Position p2 = ledger.addEntry("dummy-entry-2".getBytes(Encoding));
         ledger.addEntry("dummy-entry-3".getBytes(Encoding));
-        PositionImpl p4 = (PositionImpl) ledger.addEntry("dummy-entry-4".getBytes(Encoding));
+        Position p4 = ledger.addEntry("dummy-entry-4".getBytes(Encoding));
 
         assertEquals(cursor.getNumberOfEntries(), 4);
 
@@ -473,7 +474,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
         assertFalse(cursor.hasMoreEntries());
         assertEquals(cursor.getNumberOfEntries(), 0);
 
-        assertEquals(cursor.getReadPosition(), new PositionImpl(p4.getLedgerId(), p4.getEntryId() + 1));
+        assertEquals(cursor.getReadPosition(), PositionFactory.create(p4.getLedgerId(), p4.getEntryId() + 1));
     }
 
     @Test(timeOut = 20000)
@@ -546,7 +547,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
     void testSingleDelete() throws Exception {
         ManagedLedger ledger = factory.open("my_test_ledger", new ManagedLedgerConfig().setMaxEntriesPerLedger(3)
                 .setRetentionTime(1, TimeUnit.HOURS).setRetentionSizeInMB(1));
-        ManagedCursor cursor = ledger.newNonDurableCursor(PositionImpl.LATEST);
+        ManagedCursor cursor = ledger.newNonDurableCursor(PositionFactory.LATEST);
 
         Position p1 = ledger.addEntry("entry1".getBytes());
         Position p2 = ledger.addEntry("entry2".getBytes());
@@ -592,9 +593,9 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
         /* Position p3 = */ ledger.addEntry("entry-3".getBytes());
 
         Thread.sleep(300);
-        ManagedCursor c1 = ledger.newNonDurableCursor(PositionImpl.EARLIEST);
-        assertEquals(c1.getReadPosition(), new PositionImpl(6, 0));
-        assertEquals(c1.getMarkDeletedPosition(), new PositionImpl(6, -1));
+        ManagedCursor c1 = ledger.newNonDurableCursor(PositionFactory.EARLIEST);
+        assertEquals(c1.getReadPosition(), PositionFactory.create(6, 0));
+        assertEquals(c1.getMarkDeletedPosition(), PositionFactory.create(6, -1));
     }
 
     @Test // (timeOut = 20000)
@@ -609,9 +610,9 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
         /* Position p5 = */ ledger.addEntry("entry-5".getBytes());
         /* Position p6 = */ ledger.addEntry("entry-6".getBytes());
 
-        ManagedCursor c1 = ledger.newNonDurableCursor(PositionImpl.EARLIEST);
+        ManagedCursor c1 = ledger.newNonDurableCursor(PositionFactory.EARLIEST);
         assertEquals(c1.getReadPosition(), p1);
-        assertEquals(c1.getMarkDeletedPosition(), new PositionImpl(3, -1));
+        assertEquals(c1.getMarkDeletedPosition(), PositionFactory.create(3, -1));
         assertEquals(c1.getNumberOfEntries(), 6);
         assertEquals(c1.getNumberOfEntriesInBacklog(false), 6);
 
@@ -674,7 +675,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
         // The slowest reader should still be the durable cursor since non-durable readers are not taken into account
         assertEquals(p3, ledger.getCursors().getSlowestReaderPosition());
 
-        PositionImpl earliestPos = new PositionImpl(-1, -2);
+        Position earliestPos = PositionFactory.create(-1, -2);
 
         ManagedCursor nonCursorEarliest = ledger.newNonDurableCursor(earliestPos, ncEarliest);
 
@@ -702,7 +703,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
         ManagedLedgerImpl ledger = (ManagedLedgerImpl) factory.open("testBacklogStatsWhenDroppingData",
                 new ManagedLedgerConfig().setMaxEntriesPerLedger(1));
         ManagedCursor c1 = ledger.openCursor("c1");
-        ManagedCursor nonDurableCursor = ledger.newNonDurableCursor(PositionImpl.EARLIEST);
+        ManagedCursor nonDurableCursor = ledger.newNonDurableCursor(PositionFactory.EARLIEST);
 
         assertEquals(nonDurableCursor.getNumberOfEntries(), 0);
         assertEquals(nonDurableCursor.getNumberOfEntriesInBacklog(true), 0);
@@ -746,7 +747,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
                 new ManagedLedgerConfig().setMaxEntriesPerLedger(1).setRetentionTime(-1, TimeUnit.SECONDS)
                         .setRetentionSizeInMB(-1));
         ManagedCursor c1 = ledger.openCursor("c1");
-        ManagedCursor nonDurableCursor = ledger.newNonDurableCursor(PositionImpl.EARLIEST);
+        ManagedCursor nonDurableCursor = ledger.newNonDurableCursor(PositionFactory.EARLIEST);
 
         List<Position> positions = new ArrayList<>();
         for (int i = 0; i < 10; i++) {
@@ -755,7 +756,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
 
         CountDownLatch latch = new CountDownLatch(10);
         for (int i = 0; i < 10; i++) {
-            ledger.asyncReadEntry((PositionImpl) positions.get(i), new AsyncCallbacks.ReadEntryCallback() {
+            ledger.asyncReadEntry(positions.get(i), new AsyncCallbacks.ReadEntryCallback() {
                 @Override
                 public void readEntryComplete(Entry entry, Object ctx) {
                     latch.countDown();
@@ -819,7 +820,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
     void deleteNonDurableCursorWithName() throws Exception {
         ManagedLedger ledger = factory.open("deleteManagedLedgerWithNonDurableCursor");
 
-        ManagedCursor c = ledger.newNonDurableCursor(PositionImpl.EARLIEST, "custom-name");
+        ManagedCursor c = ledger.newNonDurableCursor(PositionFactory.EARLIEST, "custom-name");
         assertEquals(Iterables.size(ledger.getCursors()), 1);
 
         ledger.deleteCursor(c.getName());
@@ -831,7 +832,7 @@ public class NonDurableCursorTest extends MockedBookKeeperTestCase {
         ManagedLedgerImpl ledger = (ManagedLedgerImpl) factory.open("testMessagesConsumedCounterInitializedCorrect",
                         new ManagedLedgerConfig().setRetentionTime(1, TimeUnit.HOURS).setRetentionSizeInMB(1));
         Position position = ledger.addEntry("1".getBytes(Encoding));
-        NonDurableCursorImpl cursor = (NonDurableCursorImpl) ledger.newNonDurableCursor(PositionImpl.EARLIEST);
+        NonDurableCursorImpl cursor = (NonDurableCursorImpl) ledger.newNonDurableCursor(PositionFactory.EARLIEST);
         cursor.delete(position);
         assertEquals(cursor.getMessagesConsumedCounter(), 1);
         assertTrue(cursor.getMessagesConsumedCounter() <= ledger.getEntriesAddedCounter());
