@@ -55,7 +55,9 @@ import javax.crypto.SecretKey;
 import javax.naming.AuthenticationException;
 import javax.servlet.http.HttpServletRequest;
 import lombok.Cleanup;
+import org.apache.commons.lang3.reflect.FieldUtils;
 import org.apache.pulsar.broker.ServiceConfiguration;
+import org.apache.pulsar.broker.authentication.metrics.AuthenticationMetricsToken;
 import org.apache.pulsar.broker.authentication.utils.AuthTokenUtils;
 import org.apache.pulsar.common.api.AuthData;
 import org.mockito.Mockito;
@@ -70,7 +72,7 @@ public class AuthenticationProviderTokenTest {
         AuthenticationProviderToken provider = new AuthenticationProviderToken();
 
         try {
-            provider.initialize(new ServiceConfiguration());
+            provider.initialize(AuthenticationProvider.Context.builder().config(new ServiceConfiguration()).build());
             fail("should have failed");
         } catch (IOException e) {
             // Expected, secret key was not defined
@@ -135,7 +137,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         try {
             provider.authenticate(new AuthenticationDataSource() {
@@ -249,7 +251,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
     }
 
     @Test
@@ -268,7 +270,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         String token = AuthTokenUtils.createToken(secretKey, SUBJECT, Optional.empty());
 
@@ -303,7 +305,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         String token = AuthTokenUtils.createToken(secretKey, SUBJECT, Optional.empty());
 
@@ -335,7 +337,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         String token = AuthTokenUtils.createToken(secretKey, SUBJECT, Optional.empty());
 
@@ -370,7 +372,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         // Use private key to generate token
         PrivateKey privateKey = AuthTokenUtils.decodePrivateKey(Decoders.BASE64.decode(privateKeyStr), SignatureAlgorithm.RS256);
@@ -413,8 +415,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
-
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         // Use private key to generate token
         PrivateKey privateKey = AuthTokenUtils.decodePrivateKey(Decoders.BASE64.decode(privateKeyStr), SignatureAlgorithm.RS256);
@@ -460,7 +461,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         // Use private key to generate token
         PrivateKey privateKey = AuthTokenUtils.decodePrivateKey(Decoders.BASE64.decode(privateKeyStr), SignatureAlgorithm.ES256);
@@ -484,8 +485,11 @@ public class AuthenticationProviderTokenTest {
     }
 
     @Test(expectedExceptions = AuthenticationException.class)
-    public void testAuthenticateWhenNoJwtPassed() throws AuthenticationException {
+    public void testAuthenticateWhenNoJwtPassed() throws Exception {
+        @Cleanup
         AuthenticationProviderToken provider = new AuthenticationProviderToken();
+        FieldUtils.writeDeclaredField(
+                provider, "authenticationMetricsToken", mock(AuthenticationMetricsToken.class), true);
         provider.authenticate(new AuthenticationDataSource() {
             @Override
             public boolean hasDataFromCommand() {
@@ -500,8 +504,11 @@ public class AuthenticationProviderTokenTest {
     }
 
     @Test(expectedExceptions = AuthenticationException.class)
-    public void testAuthenticateWhenAuthorizationHeaderNotExist() throws AuthenticationException {
+    public void testAuthenticateWhenAuthorizationHeaderNotExist() throws Exception {
+        @Cleanup
         AuthenticationProviderToken provider = new AuthenticationProviderToken();
+        FieldUtils.writeDeclaredField(
+                provider, "authenticationMetricsToken", mock(AuthenticationMetricsToken.class), true);
         provider.authenticate(new AuthenticationDataSource() {
             @Override
             public String getHttpHeader(String name) {
@@ -516,8 +523,11 @@ public class AuthenticationProviderTokenTest {
     }
 
     @Test(expectedExceptions = AuthenticationException.class)
-    public void testAuthenticateWhenAuthHeaderValuePrefixIsInvalid() throws AuthenticationException {
+    public void testAuthenticateWhenAuthHeaderValuePrefixIsInvalid() throws Exception {
+        @Cleanup
         AuthenticationProviderToken provider = new AuthenticationProviderToken();
+        FieldUtils.writeDeclaredField(
+                provider, "authenticationMetricsToken", mock(AuthenticationMetricsToken.class), true);
         provider.authenticate(new AuthenticationDataSource() {
             @Override
             public String getHttpHeader(String name) {
@@ -532,8 +542,11 @@ public class AuthenticationProviderTokenTest {
     }
 
     @Test(expectedExceptions = AuthenticationException.class)
-    public void testAuthenticateWhenJwtIsBlank() throws AuthenticationException {
+    public void testAuthenticateWhenJwtIsBlank() throws Exception {
+        @Cleanup
         AuthenticationProviderToken provider = new AuthenticationProviderToken();
+        FieldUtils.writeDeclaredField(
+                provider, "authenticationMetricsToken", mock(AuthenticationMetricsToken.class), true);
         provider.authenticate(new AuthenticationDataSource() {
             @Override
             public String getHttpHeader(String name) {
@@ -559,7 +572,7 @@ public class AuthenticationProviderTokenTest {
         conf.setProperties(properties);
 
         AuthenticationProviderToken provider = new AuthenticationProviderToken();
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
         provider.authenticate(new AuthenticationDataSource() {
             @Override
             public String getHttpHeader(String name) {
@@ -582,7 +595,7 @@ public class AuthenticationProviderTokenTest {
         conf.setProperties(properties);
 
         AuthenticationProviderToken provider = new AuthenticationProviderToken();
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
     }
 
     @Test(expectedExceptions = IOException.class)
@@ -594,7 +607,7 @@ public class AuthenticationProviderTokenTest {
         conf.setProperties(properties);
 
         AuthenticationProviderToken provider = new AuthenticationProviderToken();
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
     }
 
     @Test(expectedExceptions = IOException.class)
@@ -606,7 +619,7 @@ public class AuthenticationProviderTokenTest {
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
 
-        new AuthenticationProviderToken().initialize(conf);
+        new AuthenticationProviderToken().initialize(AuthenticationProvider.Context.builder().config(conf).build());
     }
 
     @Test(expectedExceptions = IOException.class)
@@ -618,7 +631,7 @@ public class AuthenticationProviderTokenTest {
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
 
-        new AuthenticationProviderToken().initialize(conf);
+        new AuthenticationProviderToken().initialize(AuthenticationProvider.Context.builder().config(conf).build());
     }
 
     @Test(expectedExceptions = IllegalArgumentException.class)
@@ -633,7 +646,7 @@ public class AuthenticationProviderTokenTest {
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
 
-        new AuthenticationProviderToken().initialize(conf);
+        new AuthenticationProviderToken().initialize(AuthenticationProvider.Context.builder().config(conf).build());
     }
 
     @Test(expectedExceptions = IOException.class)
@@ -645,7 +658,7 @@ public class AuthenticationProviderTokenTest {
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
 
-        new AuthenticationProviderToken().initialize(conf);
+        new AuthenticationProviderToken().initialize(AuthenticationProvider.Context.builder().config(conf).build());
     }
 
     @Test(expectedExceptions = IllegalArgumentException.class)
@@ -657,7 +670,7 @@ public class AuthenticationProviderTokenTest {
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
 
-        new AuthenticationProviderToken().initialize(conf);
+        new AuthenticationProviderToken().initialize(AuthenticationProvider.Context.builder().config(conf).build());
     }
 
 
@@ -676,7 +689,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         // Create a token that will expire in 3 seconds
         String expiringToken = AuthTokenUtils.createToken(secretKey, SUBJECT,
@@ -709,7 +722,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         // Create a token that is already expired
         String expiringToken = AuthTokenUtils.createToken(secretKey, SUBJECT,
@@ -828,7 +841,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         // Use private key to generate token
         PrivateKey privateKey = AuthTokenUtils.decodePrivateKey(Decoders.BASE64.decode(privateKeyStr), SignatureAlgorithm.RS256);
@@ -877,7 +890,7 @@ public class AuthenticationProviderTokenTest {
                 );
         Mockito.when(mockConf.getProperty(AuthenticationProviderToken.CONF_TOKEN_SETTING_PREFIX)).thenReturn(prefix);
 
-        provider.initialize(mockConf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(mockConf).build());
 
         // Each property is fetched only once. Prevent multiple fetches.
         Mockito.verify(mockConf, Mockito.times(1)).getProperty(AuthenticationProviderToken.CONF_TOKEN_SETTING_PREFIX);
@@ -908,7 +921,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         String token = AuthTokenUtils.createToken(secretKey, SUBJECT, Optional.empty());
         HttpServletRequest servletRequest = mock(HttpServletRequest.class);
@@ -934,7 +947,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         String token = AuthTokenUtils.createToken(secretKey, SUBJECT, Optional.empty());
         HttpServletRequest servletRequest = mock(HttpServletRequest.class);
@@ -960,7 +973,7 @@ public class AuthenticationProviderTokenTest {
 
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         AuthenticationState authState = provider.newAuthState(null,null, null);
 
@@ -1016,7 +1029,7 @@ public class AuthenticationProviderTokenTest {
         properties.setProperty(AuthenticationProviderToken.CONF_TOKEN_SECRET_KEY, secretKeyFile.toString());
         ServiceConfiguration conf = new ServiceConfiguration();
         conf.setProperties(properties);
-        provider.initialize(conf);
+        provider.initialize(AuthenticationProvider.Context.builder().config(conf).build());
 
         String token = createTokenWithAudience(secretKey, audienceClaim, audiences);
 
