@@ -71,6 +71,13 @@ public abstract class AbstractBaseDispatcher extends EntryFilterSupport implemen
     private final LongAdder filterRejectedMsgs = new LongAdder();
     private final LongAdder filterRescheduledMsgs = new LongAdder();
 
+    private final LongAdder dispatchThrottledMsgEventsBySubscriptionLimit = new LongAdder();
+    private final LongAdder dispatchThrottledMsgEventsByTopicLimit = new LongAdder();
+    private final LongAdder dispatchThrottledMsgEventsByBrokerLimit = new LongAdder();
+    private final LongAdder dispatchThrottledBytesEventsBySubscriptionLimit = new LongAdder();
+    private final LongAdder dispatchThrottledBytesEventsByTopicLimit = new LongAdder();
+    private final LongAdder dispatchThrottledBytesEventsByBrokerLimit = new LongAdder();
+
     protected AbstractBaseDispatcher(Subscription subscription, ServiceConfiguration serviceConfig) {
         super(subscription);
         this.serviceConfig = serviceConfig;
@@ -405,6 +412,8 @@ public abstract class AbstractBaseDispatcher extends EntryFilterSupport implemen
     private boolean applyDispatchRateLimitsToReadLimits(DispatchRateLimiter rateLimiter,
                                                         MutablePair<Integer, Long> readLimits,
                                                         DispatchRateLimiter.Type limiterType) {
+        int originalMessagesToRead = readLimits.getLeft();
+        long originalBytesToRead = readLimits.getRight();
         // update messagesToRead according to available dispatch rate limit.
         int availablePermitsOnMsg = (int) rateLimiter.getAvailableDispatchRateLimitOnMsg();
         if (availablePermitsOnMsg >= 0) {
@@ -414,6 +423,22 @@ public abstract class AbstractBaseDispatcher extends EntryFilterSupport implemen
         if (availablePermitsOnByte >= 0) {
             readLimits.setRight(Math.min(readLimits.getRight(), availablePermitsOnByte));
         }
+        if (readLimits.getLeft() < originalMessagesToRead) {
+            switch (limiterType) {
+                case BROKER -> dispatchThrottledMsgEventsByBrokerLimit.increment();
+                case TOPIC -> dispatchThrottledMsgEventsByTopicLimit.increment();
+                case SUBSCRIPTION -> dispatchThrottledMsgEventsBySubscriptionLimit.increment();
+                default -> {}
+            }
+        }
+        if (readLimits.getRight() < originalBytesToRead) {
+            switch (limiterType) {
+                case BROKER -> dispatchThrottledBytesEventsByBrokerLimit.increment();
+                case TOPIC -> dispatchThrottledBytesEventsByTopicLimit.increment();
+                case SUBSCRIPTION -> dispatchThrottledBytesEventsBySubscriptionLimit.increment();
+                default -> {}
+            }
+        }
         if (readLimits.getLeft() == 0 || readLimits.getRight() == 0) {
             if (log.isDebugEnabled()) {
                 log.debug("[{}] message-read exceeded {} message-rate {}/{}, schedule after {}ms", getName(),
@@ -470,6 +495,36 @@ public abstract class AbstractBaseDispatcher extends EntryFilterSupport implemen
         return this.filterRescheduledMsgs.longValue();
     }
 
+    @Override
+    public long getDispatchThrottledMsgEventsBySubscriptionLimit() {
+        return dispatchThrottledMsgEventsBySubscriptionLimit.longValue();
+    }
+
+    @Override
+    public long getDispatchThrottledBytesBySubscriptionLimit() {
+        return dispatchThrottledBytesEventsBySubscriptionLimit.longValue();
+    }
+
+    @Override
+    public long getDispatchThrottledMsgEventsByTopicLimit() {
+        return dispatchThrottledMsgEventsByTopicLimit.longValue();
+    }
+
+    @Override
+    public long getDispatchThrottledBytesEventsByTopicLimit() {
+        return dispatchThrottledBytesEventsByTopicLimit.longValue();
+    }
+
+    @Override
+    public long getDispatchThrottledMsgEventsByBrokerLimit() {
+        return dispatchThrottledMsgEventsByBrokerLimit.longValue();
+    }
+
+    @Override
+    public long getDispatchThrottledBytesEventsByBrokerLimit() {
+        return dispatchThrottledBytesEventsByBrokerLimit.longValue();
+    }
+
     protected final void updatePendingBytesToDispatch(long size) {
         PENDING_BYTES_TO_DISPATCH.inc(size);
     }
