@@ -20,20 +20,21 @@ package org.apache.pulsar.broker.service;
 
 import com.google.common.annotations.VisibleForTesting;
 import io.netty.buffer.ByteBuf;
-import java.nio.charset.StandardCharsets;
+import java.util.function.ToIntFunction;
+import javax.annotation.Nullable;
 import lombok.Getter;
 import org.apache.bookkeeper.mledger.Entry;
 import org.apache.bookkeeper.mledger.Position;
 import org.apache.pulsar.common.api.proto.MessageMetadata;
 import org.apache.pulsar.common.protocol.Commands;
-import org.jspecify.annotations.Nullable;
 
 public class EntryAndMetadata implements Entry {
-
+    private static final int STICKY_KEY_HASH_NOT_INITIALIZED = -1;
     private final Entry entry;
     @Getter
     @Nullable
     private final MessageMetadata metadata;
+    int stickyKeyHash = STICKY_KEY_HASH_NOT_INITIALIZED;
 
     private EntryAndMetadata(final Entry entry, @Nullable final MessageMetadata metadata) {
         this.entry = entry;
@@ -51,13 +52,9 @@ public class EntryAndMetadata implements Entry {
 
     public byte[] getStickyKey() {
         if (metadata != null) {
-            if (metadata.hasOrderingKey()) {
-                return metadata.getOrderingKey();
-            } else if (metadata.hasPartitionKey()) {
-                return metadata.getPartitionKey().getBytes(StandardCharsets.UTF_8);
-            }
+            return Commands.resolveStickyKey(metadata);
         }
-        return "NONE_KEY".getBytes(StandardCharsets.UTF_8);
+        return Commands.NONE_KEY;
     }
 
     @Override
@@ -111,4 +108,32 @@ public class EntryAndMetadata implements Entry {
     public boolean release() {
         return entry.release();
     }
+
+    /**
+     * Get cached sticky key hash or calculate it based on the sticky key if it's not cached.
+     *
+     * @param makeStickyKeyHash function to calculate the sticky key hash
+     * @return the sticky key hash
+     */
+    public int getOrUpdateCachedStickyKeyHash(ToIntFunction<byte[]> makeStickyKeyHash) {
+        if (stickyKeyHash == STICKY_KEY_HASH_NOT_INITIALIZED) {
+            stickyKeyHash = makeStickyKeyHash.applyAsInt(getStickyKey());
+        }
+        return stickyKeyHash;
+    }
+
+    /**
+     * Get cached sticky key hash or return STICKY_KEY_HASH_NOT_SET if it's not cached.
+     *
+     * @return the cached sticky key hash or STICKY_KEY_HASH_NOT_SET if it's not cached
+     */
+    public int getCachedStickyKeyHash() {
+        return stickyKeyHash != STICKY_KEY_HASH_NOT_INITIALIZED ? stickyKeyHash
+                : StickyKeyConsumerSelector.STICKY_KEY_HASH_NOT_SET;
+    }
+
+    @VisibleForTesting
+    public Entry unwrap() {
+        return entry;
+    }
 }
