@@ -19,12 +19,12 @@
 package org.apache.pulsar.broker.service;
 
 import com.google.common.collect.Sets;
+import com.google.common.io.Resources;
 import io.netty.channel.Channel;
 import java.net.URL;
 import java.nio.channels.SelectionKey;
 import java.util.Collections;
 import java.util.Optional;
-import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.atomic.AtomicBoolean;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.pulsar.broker.PulsarService;
@@ -34,20 +34,24 @@ import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.common.policies.data.ClusterData;
 import org.apache.pulsar.common.policies.data.TenantInfoImpl;
 import org.apache.pulsar.common.policies.data.TopicType;
-import org.apache.pulsar.metadata.api.extended.SessionEvent;
 import org.apache.pulsar.metadata.impl.ZKMetadataStore;
 import org.apache.pulsar.tests.TestRetrySupport;
 import org.apache.pulsar.zookeeper.LocalBookkeeperEnsemble;
 import org.apache.pulsar.zookeeper.ZookeeperServerTest;
 import org.apache.zookeeper.ClientCnxn;
 import org.apache.zookeeper.ZooKeeper;
-import org.awaitility.Awaitility;
 import org.awaitility.reflect.WhiteboxImpl;
 
 @Slf4j
 public abstract class CanReconnectZKClientPulsarServiceBaseTest extends TestRetrySupport {
     protected final String defaultTenant = "public";
     protected final String defaultNamespace = defaultTenant + "/default";
+    final static String caCertPath = Resources.getResource("certificate-authority/certs/ca.cert.pem")
+            .getPath();
+    final static String brokerCertPath =
+            Resources.getResource("certificate-authority/server-keys/broker.cert.pem").getPath();
+    final static String brokerKeyPath =
+            Resources.getResource("certificate-authority/server-keys/broker.key-pk8.pem").getPath();
     protected int numberOfBookies = 3;
     protected final String clusterName = "r1";
     protected URL url;
@@ -60,10 +64,8 @@ public abstract class CanReconnectZKClientPulsarServiceBaseTest extends TestRetr
     protected PulsarAdmin admin;
     protected PulsarClient client;
     protected ZooKeeper localZkOfBroker;
-    protected volatile SessionEvent sessionEvent;
     protected Object localMetaDataStoreClientCnx;
-    protected final AtomicBoolean connectionTerminationThreadKeepRunning = new AtomicBoolean();
-    private volatile Thread connectionTerminationThread;
+    protected final AtomicBoolean LocalMetadataStoreInReconnectFinishSignal = new AtomicBoolean();
 
     protected void startZKAndBK() throws Exception {
         // Start ZK.
@@ -83,10 +85,6 @@ public abstract class CanReconnectZKClientPulsarServiceBaseTest extends TestRetr
         broker = pulsar.getBrokerService();
         ZKMetadataStore zkMetadataStore = (ZKMetadataStore) pulsar.getLocalMetadataStore();
         localZkOfBroker = zkMetadataStore.getZkClient();
-        zkMetadataStore.registerSessionListener(n -> {
-            log.info("Received session event: {}", n);
-            sessionEvent = n;
-        });
         ClientCnxn cnxn = WhiteboxImpl.getInternalState(localZkOfBroker, "cnxn");
         Object sendThread = WhiteboxImpl.getInternalState(cnxn, "sendThread");
         localMetaDataStoreClientCnx = WhiteboxImpl.getInternalState(sendThread, "clientCnxnSocket");
@@ -97,29 +95,25 @@ public abstract class CanReconnectZKClientPulsarServiceBaseTest extends TestRetr
         client = PulsarClient.builder().serviceUrl(url.toString()).build();
     }
 
-    protected void startLocalMetadataStoreConnectionTermination() throws Exception {
-        if (!connectionTerminationThreadKeepRunning.compareAndSet(false, true)) {
-            throw new RuntimeException("Local metadata store connection is already being terminated");
+    protected void makeLocalMetadataStoreKeepReconnect() throws Exception {
+        if (!LocalMetadataStoreInReconnectFinishSignal.compareAndSet(false, true)) {
+            throw new RuntimeException("Local metadata store is already keeping reconnect");
         }
-        CompletableFuture<Void> future = new CompletableFuture<>();
         if (localMetaDataStoreClientCnx.getClass().getSimpleName().equals("ClientCnxnSocketNIO")) {
-            startNIOImplTermination(future);
+            makeLocalMetadataStoreKeepReconnectNIO();
         } else {
             // ClientCnxnSocketNetty.
-            startNettyImplTermination(future);
+            makeLocalMetadataStoreKeepReconnectNetty();
         }
-        // wait until connection is closed at least once
-        future.get();
     }
 
-    private void startNIOImplTermination(CompletableFuture<Void> future) {
-        connectionTerminationThread = new Thread(() -> {
-            while (connectionTerminationThreadKeepRunning.get()) {
+    protected void makeLocalMetadataStoreKeepReconnectNIO() {
+        new Thread(() -> {
+            while (LocalMetadataStoreInReconnectFinishSignal.get()) {
                 try {
                     SelectionKey sockKey = WhiteboxImpl.getInternalState(localMetaDataStoreClientCnx, "sockKey");
                     if (sockKey != null) {
                         sockKey.channel().close();
-                        future.complete(null);
                     }
                     // Prevents high cpu usage.
                     Thread.sleep(5);
@@ -127,18 +121,16 @@ public abstract class CanReconnectZKClientPulsarServiceBaseTest extends TestRetr
                     log.error("Try close the ZK connection of local metadata store failed: {}", e.toString());
                 }
             }
-        });
-        connectionTerminationThread.start();
+        }).start();
     }
 
-    private void startNettyImplTermination(CompletableFuture<Void> future) {
-        connectionTerminationThread = new Thread(() -> {
-            while (connectionTerminationThreadKeepRunning.get()) {
+    protected void makeLocalMetadataStoreKeepReconnectNetty() {
+        new Thread(() -> {
+            while (LocalMetadataStoreInReconnectFinishSignal.get()) {
                 try {
                     Channel channel = WhiteboxImpl.getInternalState(localMetaDataStoreClientCnx, "channel");
                     if (channel != null) {
                         channel.close();
-                        future.complete(null);
                     }
                     // Prevents high cpu usage.
                     Thread.sleep(5);
@@ -146,18 +138,11 @@ public abstract class CanReconnectZKClientPulsarServiceBaseTest extends TestRetr
                     log.error("Try close the ZK connection of local metadata store failed: {}", e.toString());
                 }
             }
-        });
-        connectionTerminationThread.start();
+        }).start();
     }
 
-    protected void stopLocalMetadataStoreConnectionTermination() throws InterruptedException {
-        connectionTerminationThreadKeepRunning.set(false);
-        if (connectionTerminationThread != null) {
-            // Wait for the reconnect thread to finish.
-            connectionTerminationThread.join();
-            connectionTerminationThread = null;
-        }
-        Awaitility.await().until(() -> SessionEvent.Reconnected.equals(sessionEvent));
+    protected void stopLocalMetadataStoreAlwaysReconnect() {
+        LocalMetadataStoreInReconnectFinishSignal.set(false);
     }
 
     protected void createDefaultTenantsAndClustersAndNamespace() throws Exception {
@@ -210,6 +195,9 @@ public abstract class CanReconnectZKClientPulsarServiceBaseTest extends TestRetr
         config.setAllowAutoTopicCreationType(TopicType.NON_PARTITIONED);
         config.setEnableReplicatedSubscriptions(true);
         config.setReplicatedSubscriptionsSnapshotFrequencyMillis(1000);
+        config.setTlsTrustCertsFilePath(caCertPath);
+        config.setTlsCertificateFilePath(brokerCertPath);
+        config.setTlsKeyFilePath(brokerKeyPath);
     }
 
     @Override
@@ -217,7 +205,7 @@ public abstract class CanReconnectZKClientPulsarServiceBaseTest extends TestRetr
         markCurrentSetupNumberCleaned();
         log.info("--- Shutting down ---");
 
-        stopLocalMetadataStoreConnectionTermination();
+        stopLocalMetadataStoreAlwaysReconnect();
 
         // Stop brokers.
         if (client != null) {
