@@ -29,16 +29,12 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 import java.util.UUID;
-import java.util.stream.Collectors;
-import org.apache.pulsar.broker.PulsarServerException;
 import org.apache.pulsar.broker.service.persistent.PersistentTopic;
 import org.apache.pulsar.client.admin.PulsarAdmin;
 import org.apache.pulsar.client.admin.PulsarAdminException;
-import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.SubscriptionType;
 import org.apache.pulsar.common.naming.TopicName;
 import org.apache.pulsar.common.policies.data.AutoSubscriptionCreationOverride;
-import org.apache.pulsar.common.policies.data.BacklogQuota;
 import org.apache.pulsar.common.policies.data.DispatchRate;
 import org.apache.pulsar.common.policies.data.DelayedDeliveryPolicies;
 import org.apache.pulsar.common.policies.data.InactiveTopicDeleteMode;
@@ -85,7 +81,6 @@ public class ReplicatorTopicPoliciesTest extends ReplicatorTestBase {
         BacklogQuotaImpl backlogQuota = new BacklogQuotaImpl();
         backlogQuota.setLimitSize(1);
         backlogQuota.setLimitTime(2);
-        backlogQuota.setPolicy(BacklogQuota.RetentionPolicy.producer_exception);
         // local
         admin1.topicPolicies().setBacklogQuota(topic, backlogQuota);
         Awaitility.await().untilAsserted(() ->
@@ -752,7 +747,7 @@ public class ReplicatorTopicPoliciesTest extends ReplicatorTestBase {
         assertNotNull(topicRef);
 
         Awaitility.await().untilAsserted(() -> {
-            List<String> replicaClusters = topicRef.getReplicators().keys().stream().sorted().collect(Collectors.toList());
+            List<String> replicaClusters = topicRef.getReplicators().keySet().stream().sorted().toList();
             assertEquals(replicaClusters.size(), 1);
             assertEquals(replicaClusters.toString(), "[r2]");
         });
@@ -760,7 +755,7 @@ public class ReplicatorTopicPoliciesTest extends ReplicatorTestBase {
         // removing topic replica cluster policy, so namespace policy should take effect
         admin1.topics().removeReplicationClusters(persistentTopicName);
         Awaitility.await().untilAsserted(() -> {
-            List<String> replicaClusters = topicRef.getReplicators().keys().stream().sorted().collect(Collectors.toList());
+            List<String> replicaClusters = topicRef.getReplicators().keySet().stream().sorted().toList();
             assertEquals(replicaClusters.size(), 2);
             assertEquals(replicaClusters.toString(), "[r2, r3]");
         });
@@ -794,8 +789,7 @@ public class ReplicatorTopicPoliciesTest extends ReplicatorTestBase {
                 assertNull(admin3.topicPolicies(true).getAutoSubscriptionCreation(topic, false)));
     }
 
-    private void init(String namespace, String topic)
-            throws PulsarAdminException, PulsarClientException, PulsarServerException {
+    private void init(String namespace, String topic) throws Exception {
         final String cluster2 = pulsar2.getConfig().getClusterName();
         final String cluster1 = pulsar1.getConfig().getClusterName();
         final String cluster3 = pulsar3.getConfig().getClusterName();
@@ -819,11 +813,9 @@ public class ReplicatorTopicPoliciesTest extends ReplicatorTestBase {
         pulsar3.getClient().newProducer().topic(topic).create().close();
 
         //init topic policies server
-        Awaitility.await().ignoreExceptions().untilAsserted(() -> {
-            assertNull(pulsar1.getTopicPoliciesService().getTopicPolicies(TopicName.get(topic)));
-            assertNull(pulsar2.getTopicPoliciesService().getTopicPolicies(TopicName.get(topic)));
-            assertNull(pulsar3.getTopicPoliciesService().getTopicPolicies(TopicName.get(topic)));
-        });
+        TopicPolicyTestUtils.getTopicPolicies(pulsar1.getTopicPoliciesService(), TopicName.get(topic));
+        TopicPolicyTestUtils.getTopicPolicies(pulsar2.getTopicPoliciesService(), TopicName.get(topic));
+        TopicPolicyTestUtils.getTopicPolicies(pulsar3.getTopicPoliciesService(), TopicName.get(topic));
     }
 
 }
