@@ -21,8 +21,8 @@ package org.apache.pulsar.client.impl;
 import static org.testng.Assert.assertEquals;
 import io.netty.channel.EventLoopGroup;
 import io.netty.channel.nio.NioEventLoopGroup;
+import io.opentelemetry.api.OpenTelemetry;
 import java.util.Collections;
-import java.util.concurrent.TimeUnit;
 import lombok.Cleanup;
 import org.apache.bookkeeper.mledger.ManagedLedger;
 import org.apache.bookkeeper.mledger.ManagedLedgerFactory;
@@ -56,30 +56,26 @@ public class SequenceIdWithErrorTest extends BkEnsemblesTestBase {
         Consumer<String> consumer = client.newConsumer(Schema.STRING).topic(topicName).subscriptionName("sub")
                 .subscribe();
 
-        // Create a producer
-        Producer<String> producer = client.newProducer(Schema.STRING).topic(topicName).create();
-        // Move the fence timing to after the first message is successfully written
-        // The current ledger is not empty, the Broker recovery will not take the abnormal path of
-        // "deleting empty ledger + unable to find old ledger"
-        producer.send("Hello-0");
-
         // Fence the topic by opening the ManagedLedger for the topic outside the Pulsar broker. This will cause the
         // broker to fail subsequent send operation and it will trigger a recover
         EventLoopGroup eventLoopGroup = new NioEventLoopGroup(1);
         ManagedLedgerClientFactory clientFactory = new ManagedLedgerClientFactory();
         clientFactory.initialize(pulsar.getConfiguration(), pulsar.getLocalMetadataStore(),
-                pulsar.getBookKeeperClientFactory(), eventLoopGroup);
-        ManagedLedgerFactory mlFactory = clientFactory.getManagedLedgerFactory();
+                pulsar.getBookKeeperClientFactory(), eventLoopGroup, OpenTelemetry.noop());
+        ManagedLedgerFactory mlFactory = clientFactory.getDefaultStorageClass().getManagedLedgerFactory();
         ManagedLedger ml = mlFactory.open(TopicName.get(topicName).getPersistenceNamingEncoding());
         ml.close();
         clientFactory.close();
 
-        for (int i = 1; i < N; i++) {
+        // Create a producer
+        Producer<String> producer = client.newProducer(Schema.STRING).topic(topicName).create();
+
+        for (int i = 0; i < N; i++) {
             producer.send("Hello-" + i);
         }
 
         for (int i = 0; i < N; i++) {
-            Message<String> msg = consumer.receive(10, TimeUnit.SECONDS);
+            Message<String> msg = consumer.receive();
             assertEquals(msg.getValue(), "Hello-" + i);
             assertEquals(msg.getSequenceId(), i);
             consumer.acknowledge(msg);
