@@ -30,10 +30,10 @@ import java.io.IOException;
 import java.util.List;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentLinkedQueue;
-import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Semaphore;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.transaction.TransactionCoordinatorClientException;
 import org.apache.pulsar.client.api.transaction.TxnID;
@@ -90,7 +90,7 @@ public class TransactionMetaStoreHandler extends HandlerState
 
     private final CompletableFuture<Void> connectFuture;
     private final long lookupDeadline;
-    private final List<Throwable> previousExceptions = new CopyOnWriteArrayList<>();
+    private final AtomicInteger previousExceptionCount = new AtomicInteger();
 
 
 
@@ -122,11 +122,11 @@ public class TransactionMetaStoreHandler extends HandlerState
     }
 
     @Override
-    public boolean connectionFailed(PulsarClientException exception) {
+    public void connectionFailed(PulsarClientException exception) {
         boolean nonRetriableError = !PulsarClientException.isRetriableError(exception);
         boolean timeout = System.currentTimeMillis() > lookupDeadline;
         if (nonRetriableError || timeout) {
-            exception.setPreviousExceptions(previousExceptions);
+            exception.setPreviousExceptionCount(previousExceptionCount);
             if (connectFuture.completeExceptionally(exception)) {
                 if (nonRetriableError) {
                     LOG.error("Transaction meta handler with transaction coordinator id {} connection failed.",
@@ -136,12 +136,10 @@ public class TransactionMetaStoreHandler extends HandlerState
                             + "timeout", transactionCoordinatorId, exception);
                 }
                 setState(State.Failed);
-                return false;
             }
         } else {
-            previousExceptions.add(exception);
+            previousExceptionCount.getAndIncrement();
         }
-        return true;
     }
 
     @Override
@@ -236,8 +234,7 @@ public class TransactionMetaStoreHandler extends HandlerState
         }
         long requestId = client.newRequestId();
         ByteBuf cmd = Commands.newTxn(transactionCoordinatorId, requestId, unit.toMillis(timeout));
-        String description = String.format("Create new transaction %s", transactionCoordinatorId);
-        OpForTxnIdCallBack op = OpForTxnIdCallBack.create(cmd, callback, client, description, cnx());
+        OpForTxnIdCallBack op = OpForTxnIdCallBack.create(cmd, callback, client);
         internalPinnedExecutor.execute(() -> {
             pendingRequests.put(requestId, op);
             timeoutQueue.add(new RequestTime(System.currentTimeMillis(), requestId));
@@ -318,10 +315,8 @@ public class TransactionMetaStoreHandler extends HandlerState
         long requestId = client.newRequestId();
         ByteBuf cmd = Commands.newAddPartitionToTxn(
                 requestId, txnID.getLeastSigBits(), txnID.getMostSigBits(), partitions);
-        String description = String.format("Add partition %s to TXN %s", String.valueOf(partitions),
-                String.valueOf(txnID));
         OpForVoidCallBack op = OpForVoidCallBack
-                .create(cmd, callback, client, description, cnx());
+                .create(cmd, callback, client);
         internalPinnedExecutor.execute(() -> {
             pendingRequests.put(requestId, op);
             timeoutQueue.add(new RequestTime(System.currentTimeMillis(), requestId));
@@ -405,9 +400,7 @@ public class TransactionMetaStoreHandler extends HandlerState
         long requestId = client.newRequestId();
         ByteBuf cmd = Commands.newAddSubscriptionToTxn(
                 requestId, txnID.getLeastSigBits(), txnID.getMostSigBits(), subscriptionList);
-        String description = String.format("Add subscription %s to TXN %s", toStringSubscriptionList(subscriptionList),
-                String.valueOf(txnID));
-        OpForVoidCallBack op = OpForVoidCallBack.create(cmd, callback, client, description, cnx());
+        OpForVoidCallBack op = OpForVoidCallBack.create(cmd, callback, client);
         internalPinnedExecutor.execute(() -> {
             pendingRequests.put(requestId, op);
             timeoutQueue.add(new RequestTime(System.currentTimeMillis(), requestId));
@@ -418,17 +411,6 @@ public class TransactionMetaStoreHandler extends HandlerState
         return callback;
     }
 
-    private String toStringSubscriptionList(List<Subscription> list) {
-        if (list == null || list.isEmpty()) {
-            return "[]";
-        }
-        StringBuilder builder = new StringBuilder("[");
-        for (Subscription subscription : list) {
-            builder.append(String.format("%s %s", subscription.getTopic(), subscription.getSubscription()));
-        }
-        return builder.append("]").toString();
-    }
-
     public void handleAddSubscriptionToTxnResponse(CommandAddSubscriptionToTxnResponse response) {
         final boolean hasError = response.hasError();
         final ServerError error;
@@ -500,8 +482,7 @@ public class TransactionMetaStoreHandler extends HandlerState
         long requestId = client.newRequestId();
         BaseCommand cmd = Commands.newEndTxn(requestId, txnID.getLeastSigBits(), txnID.getMostSigBits(), action);
         ByteBuf buf = Commands.serializeWithSize(cmd);
-        String description = String.format("End [%s] TXN %s", String.valueOf(action), String.valueOf(txnID));
-        OpForVoidCallBack op = OpForVoidCallBack.create(buf, callback, client, description, cnx());
+        OpForVoidCallBack op = OpForVoidCallBack.create(buf, callback, client);
         internalPinnedExecutor.execute(() -> {
             pendingRequests.put(requestId, op);
             timeoutQueue.add(new RequestTime(System.currentTimeMillis(), requestId));
@@ -591,16 +572,13 @@ public class TransactionMetaStoreHandler extends HandlerState
         protected ByteBuf cmd;
         protected CompletableFuture<T> callback;
         protected Backoff backoff;
-        protected String description;
-        protected ClientCnx clientCnx;
 
         abstract void recycle();
     }
 
     private static class OpForTxnIdCallBack extends OpBase<TxnID> {
 
-        static OpForTxnIdCallBack create(ByteBuf cmd, CompletableFuture<TxnID> callback, PulsarClientImpl client,
-                                         String description, ClientCnx clientCnx) {
+        static OpForTxnIdCallBack create(ByteBuf cmd, CompletableFuture<TxnID> callback, PulsarClientImpl client) {
             OpForTxnIdCallBack op = RECYCLER.get();
             op.callback = callback;
             op.cmd = cmd;
@@ -610,8 +588,6 @@ public class TransactionMetaStoreHandler extends HandlerState
                     .setMax(client.getConfiguration().getMaxBackoffIntervalNanos() / 10, TimeUnit.NANOSECONDS)
                     .setMandatoryStop(0, TimeUnit.MILLISECONDS)
                     .create();
-            op.description = description;
-            op.clientCnx = clientCnx;
             return op;
         }
 
@@ -624,8 +600,6 @@ public class TransactionMetaStoreHandler extends HandlerState
             this.backoff = null;
             this.cmd = null;
             this.callback = null;
-            this.description = null;
-            this.clientCnx = null;
             recyclerHandle.recycle(this);
         }
 
@@ -641,8 +615,7 @@ public class TransactionMetaStoreHandler extends HandlerState
     private static class OpForVoidCallBack extends OpBase<Void> {
 
 
-        static OpForVoidCallBack create(ByteBuf cmd, CompletableFuture<Void> callback, PulsarClientImpl client,
-                                        String description, ClientCnx clientCnx) {
+        static OpForVoidCallBack create(ByteBuf cmd, CompletableFuture<Void> callback, PulsarClientImpl client) {
             OpForVoidCallBack op = RECYCLER.get();
             op.callback = callback;
             op.cmd = cmd;
@@ -652,8 +625,6 @@ public class TransactionMetaStoreHandler extends HandlerState
                     .setMax(client.getConfiguration().getMaxBackoffIntervalNanos() / 10, TimeUnit.NANOSECONDS)
                     .setMandatoryStop(0, TimeUnit.MILLISECONDS)
                     .create();
-            op.description = description;
-            op.clientCnx = clientCnx;
             return op;
         }
 
@@ -666,8 +637,6 @@ public class TransactionMetaStoreHandler extends HandlerState
             this.backoff = null;
             this.cmd = null;
             this.callback = null;
-            this.description = null;
-            this.clientCnx = null;
             recyclerHandle.recycle(this);
         }
 
@@ -775,8 +744,7 @@ public class TransactionMetaStoreHandler extends HandlerState
                     OpBase<?> op = pendingRequests.remove(lastPolled.requestId);
                     if (op != null && !op.callback.isDone()) {
                         op.callback.completeExceptionally(new PulsarClientException.TimeoutException(
-                            String.format("%s failed due to timeout. connection: %s. pending-queue: %s",
-                                op.description, op.clientCnx, pendingRequests.size())));
+                                "Could not get response from transaction meta store within given timeout."));
                         if (LOG.isDebugEnabled()) {
                             LOG.debug("Transaction coordinator request {} is timeout.", lastPolled.requestId);
                         }
