@@ -24,6 +24,7 @@ import static org.mockito.Mockito.never;
 import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.when;
 import com.google.common.collect.Sets;
+import com.google.re2j.Pattern;
 import org.apache.pulsar.common.lookup.GetTopicsResult;
 import org.testng.annotations.BeforeMethod;
 import org.testng.annotations.Test;
@@ -32,7 +33,6 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.concurrent.CompletableFuture;
 import java.util.function.Consumer;
-import java.util.regex.Pattern;
 
 public class PatternMultiTopicsConsumerImplTest {
 
@@ -61,7 +61,7 @@ public class PatternMultiTopicsConsumerImplTest {
                         "persistent://tenant/my-ns/non-matching"),
                         null, false, true),
                 mockListener,
-                Collections.emptyList());
+                Collections.emptyList(), "");
         verify(mockListener).onTopicsAdded(Sets.newHashSet(
                 "persistent://tenant/my-ns/name-1",
                 "persistent://tenant/my-ns/name-2"));
@@ -80,7 +80,7 @@ public class PatternMultiTopicsConsumerImplTest {
                         "persistent://tenant/my-ns/name-2"),
                         "TOPICS_HASH", true, true),
                 mockListener,
-                Arrays.asList("persistent://tenant/my-ns/name-0"));
+                Arrays.asList("persistent://tenant/my-ns/name-0"), "");
         verify(mockListener).onTopicsAdded(Sets.newHashSet(
                 "persistent://tenant/my-ns/name-1",
                 "persistent://tenant/my-ns/name-2"));
@@ -99,7 +99,7 @@ public class PatternMultiTopicsConsumerImplTest {
                         "persistent://tenant/my-ns/name-2"),
                         "TOPICS_HASH", true, false),
                 mockListener,
-                Arrays.asList("persistent://tenant/my-ns/name-0"));
+                Arrays.asList("persistent://tenant/my-ns/name-0"), "");
         verify(mockListener, never()).onTopicsAdded(any());
         verify(mockListener, never()).onTopicsRemoved(any());
         verify(mockTopicsHashSetter).accept("TOPICS_HASH");
