@@ -122,7 +122,7 @@ public abstract class PulsarDecoder extends ChannelInboundHandlerAdapter {
             cmd.parseFrom(buffer, cmdSize);
 
             if (log.isDebugEnabled()) {
-                log.debug("[{}] Received cmd {}", ctx.channel().remoteAddress(), cmd.getType());
+                log.debug("[{}] Received cmd {}", ctx.channel(), cmd.getType());
             }
             messageReceived();
 
