@@ -251,7 +251,7 @@ public class PerformanceClient extends CmdBase {
                     Authentication auth = AuthenticationFactory.create(this.authPluginClassName,
                             this.authParams);
                     auth.start();
-                    AuthenticationDataProvider authData = auth.getAuthData();
+                    AuthenticationDataProvider authData = auth.getAuthData(produceUri.getHost());
                     if (authData.hasDataForHttp()) {
                         for (Map.Entry<String, String> kv : authData.getHttpHeaders()) {
                             produceRequest.setHeader(kv.getKey(), kv.getValue());
@@ -259,9 +259,6 @@ public class PerformanceClient extends CmdBase {
                     }
                 } catch (Exception e) {
                     log.error("Authentication plugin error: " + e.getMessage());
-                    if (PerfClientUtils.hasInterruptedException(e)) {
-                        Thread.currentThread().interrupt();
-                    }
                 }
             }
 
@@ -275,9 +272,6 @@ public class PerformanceClient extends CmdBase {
                 return;
             } catch (Exception e1) {
                 log.error("Fail in starting client[{}]", e1.getMessage());
-                if (PerfClientUtils.hasInterruptedException(e1)) {
-                    Thread.currentThread().interrupt();
-                }
                 return;
             }
 
@@ -294,7 +288,7 @@ public class PerformanceClient extends CmdBase {
                 long testEndTime = startTime + (long) (this.testTime * 1e9);
                 // Send messages on all topics/producers
                 long totalSent = 0;
-                while (!Thread.currentThread().isInterrupted()) {
+                while (true) {
                     for (String topic : producersMap.keySet()) {
                         if (this.testTime > 0 && System.nanoTime() > testEndTime) {
                             log.info("------------- DONE (reached the maximum duration: [{} seconds] of production) "
@@ -358,11 +352,10 @@ public class PerformanceClient extends CmdBase {
         histogramLogWriter.outputLogFormatVersion();
         histogramLogWriter.outputLegend();
 
-        while (!Thread.currentThread().isInterrupted()) {
+        while (true) {
             try {
                 Thread.sleep(5000);
             } catch (InterruptedException e) {
-                Thread.currentThread().interrupt();
                 break;
             }
 
@@ -406,9 +399,6 @@ public class PerformanceClient extends CmdBase {
             Class clz = classLoader.loadClass(formatterClass);
             return (IMessageFormatter) clz.getDeclaredConstructor().newInstance();
         } catch (Exception e) {
-            if (PerfClientUtils.hasInterruptedException(e)) {
-                Thread.currentThread().interrupt();
-            }
             return null;
         }
     }
@@ -418,12 +408,11 @@ public class PerformanceClient extends CmdBase {
         loadArguments();
         PerfClientUtils.printJVMInformation(log);
         long start = System.nanoTime();
-        Thread shutdownHookThread = PerfClientUtils.addShutdownHook(() -> {
+        Runtime.getRuntime().addShutdownHook(new Thread(() -> {
             printAggregatedThroughput(start);
             printAggregatedStats();
-        });
+        }));
         runPerformanceTest();
-        PerfClientUtils.removeAndRunShutdownHook(shutdownHookThread);
     }
 
     private class Tuple {
