@@ -20,7 +20,7 @@ package org.apache.pulsar.testclient;
 
 import static org.apache.pulsar.broker.loadbalance.extensions.ExtensibleLoadManagerImpl.BROKER_LOAD_DATA_STORE_TOPIC;
 import static org.apache.pulsar.broker.resources.LoadBalanceResources.BROKER_TIME_AVERAGE_BASE_PATH;
-import com.google.gson.Gson;
+import com.fasterxml.jackson.databind.ObjectReader;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -35,6 +35,8 @@ import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.Schema;
 import org.apache.pulsar.client.api.SizeUnit;
 import org.apache.pulsar.client.api.TableView;
+import org.apache.pulsar.common.util.ObjectMapperFactory;
+import org.apache.pulsar.policies.data.loadbalancer.LoadManagerReport;
 import org.apache.pulsar.policies.data.loadbalancer.LoadReport;
 import org.apache.pulsar.policies.data.loadbalancer.LocalBrokerData;
 import org.apache.pulsar.policies.data.loadbalancer.ResourceUsage;
@@ -63,7 +65,11 @@ public class BrokerMonitor extends CmdBase {
     private static final int ZOOKEEPER_TIMEOUT_MILLIS = 30000;
     private static final int GLOBAL_STATS_PRINT_PERIOD_MILLIS = 60000;
     private ZooKeeper zkClient;
-    private static final Gson gson = new Gson();
+    private static final ObjectReader LOAD_REPORT_READER = ObjectMapperFactory.getMapper().reader()
+            .forType(LoadManagerReport.class);
+
+    private static final ObjectReader TIME_AVERAGE_READER = ObjectMapperFactory.getMapper().reader()
+            .forType(TimeAverageBrokerData.class);
 
     // Fields common for message rows.
     private static final List<Object> MESSAGE_FIELDS = Arrays.asList("MSG/S IN", "MSG/S OUT", "TOTAL", "KB/S IN",
@@ -85,7 +91,7 @@ public class BrokerMonitor extends CmdBase {
     private static final Object[] ALLOC_MESSAGE_ROW = makeMessageRow("ALLOC MSG");
     private static final Object[] GLOBAL_HEADER = { "BROKER", "BUNDLE", "MSG/S", "LONG/S", "KB/S", "MAX %" };
 
-    private Map<String, Object> loadData;
+    private Map<String, LoadManagerReport> loadData;
 
     private static final FixedColumnLengthTableMaker localTableMaker = new FixedColumnLengthTableMaker();
 
@@ -146,9 +152,9 @@ public class BrokerMonitor extends CmdBase {
             double totalLongTermMessageRate = 0;
             double maxMaxUsage = 0;
             int i = 1;
-            for (final Map.Entry<String, Object> entry : loadData.entrySet()) {
+            for (final Map.Entry<String, LoadManagerReport> entry : loadData.entrySet()) {
                 final String broker = entry.getKey();
-                final Object data = entry.getValue();
+                final LoadManagerReport data = entry.getValue();
                 rows[i] = new Object[GLOBAL_HEADER.length];
                 rows[i][0] = broker;
                 int numBundles;
@@ -177,9 +183,8 @@ public class BrokerMonitor extends CmdBase {
                     messageRate = localData.getMsgRateIn() + localData.getMsgRateOut();
                     final String timeAveragePath = BROKER_TIME_AVERAGE_BASE_PATH + "/" + broker;
                     try {
-                        final TimeAverageBrokerData timeAverageData = gson.fromJson(
-                                new String(zkClient.getData(timeAveragePath, false, null)),
-                                TimeAverageBrokerData.class);
+                        final TimeAverageBrokerData timeAverageData = TIME_AVERAGE_READER.readValue(
+                                new String(zkClient.getData(timeAveragePath, false, null)));
                         longTermMessageRate = timeAverageData.getLongTermMsgRateIn()
                                 + timeAverageData.getLongTermMsgRateOut();
                     } catch (Exception x) {
@@ -307,20 +312,21 @@ public class BrokerMonitor extends CmdBase {
         private synchronized void printData(final String path) {
             final String broker = brokerNameFromPath(path);
             String jsonString;
+            LoadManagerReport loadManagerReport;
             try {
                 jsonString = new String(zkClient.getData(path, this, null));
+                loadManagerReport = LOAD_REPORT_READER.readValue(jsonString);
             } catch (Exception ex) {
                 throw new RuntimeException(ex);
             }
-            // Use presence of the String "allocated" to determine if this is using SimpleLoadManagerImpl.
-            if (jsonString.contains("allocated")) {
-                printLoadReport(broker, gson.fromJson(jsonString, LoadReport.class));
-            } else {
-                final LocalBrokerData localBrokerData = gson.fromJson(jsonString, LocalBrokerData.class);
+            if (loadManagerReport instanceof LoadReport) {
+                printLoadReport(broker, (LoadReport) loadManagerReport);
+            } else  {
+                final LocalBrokerData localBrokerData = (LocalBrokerData) loadManagerReport;
                 final String timeAveragePath = BROKER_TIME_AVERAGE_BASE_PATH + "/" + broker;
                 try {
-                    final TimeAverageBrokerData timeAverageData = gson.fromJson(
-                            new String(zkClient.getData(timeAveragePath, false, null)), TimeAverageBrokerData.class);
+                    final TimeAverageBrokerData timeAverageData = TIME_AVERAGE_READER.readValue(
+                            new String(zkClient.getData(timeAveragePath, false, null)));
                     printBrokerData(broker, localBrokerData, timeAverageData);
                 } catch (Exception e) {
                     throw new RuntimeException(e);
@@ -466,7 +472,7 @@ public class BrokerMonitor extends CmdBase {
         try {
             final BrokerWatcher brokerWatcher = new BrokerWatcher(zkClient);
             brokerWatcher.updateBrokers(BROKER_ROOT);
-            while (!Thread.currentThread().isInterrupted()) {
+            while (true) {
                 Thread.sleep(GLOBAL_STATS_PRINT_PERIOD_MILLIS);
                 printGlobalData();
             }
@@ -532,7 +538,7 @@ public class BrokerMonitor extends CmdBase {
 
     private void startBrokerLoadDataStoreMonitor() {
         try {
-            while (!Thread.currentThread().isInterrupted()) {
+            while (true) {
                 Thread.sleep(GLOBAL_STATS_PRINT_PERIOD_MILLIS);
                 printBrokerLoadDataStore();
             }
